/* 
* complex_proy(Complex,AgentsWithPos,NewComplex)
* 
* Filters Complex on Agents, the simplices remaining are the ones with no other colors but the ones in Agents
*/

complex_proy(Complex,AgentsWithPos,NewComplex):-
    findall(Agent,(member(Simplex,Complex),member(v(Agent,_),Simplex)),Agents_),
    list_to_set(Agents_,Agents),
    length(Agents,NumberOfAgents),
    findall(AgentToPreserve,member(AgentToPreserve=>_,AgentsWithPos),AgentsToPreserve_),
    list_to_set(AgentsToPreserve_,AgentsToPreserve),
    findall(Pos,member(_=>Pos,AgentsWithPos),Poss_),
    list_to_set(Poss_,Poss),
    complex_proy(Complex,NumberOfAgents,AgentsToPreserve,Poss,NewComplex_),
    list_to_set(NewComplex_,NewComplex).
    
    

complex_proy([],_,_,_,[]):-!.

complex_proy([Simplex|MoreSimplices],OldNumberOfAgents,AgentsToPreserve,Poss,[NewSimplex|MoreSimplicesNewComplex]):-
    valid_simplex(Simplex,AgentsToPreserve),
    preserve_valid_info(Simplex,OldNumberOfAgents,Poss,NewSimplex),
    complex_proy(MoreSimplices,OldNumberOfAgents,AgentsToPreserve,Poss,MoreSimplicesNewComplex),
    !.
    
complex_proy([_|MoreSimplices],OldNumberOfAgents,AgentsToPreserve,Poss,MoreSimplicesNewComplex):-
    complex_proy(MoreSimplices,OldNumberOfAgents,AgentsToPreserve,Poss,MoreSimplicesNewComplex).
    
preserve_valid_info([],_,_,[]):-!.
    
preserve_valid_info([v(Color,View)|MoreVertices],OldNumberOfAgents,Poss,[v(Color,NewView)|MoreNewVertices]):-
    generate_new_view(View,OldNumberOfAgents,Poss,0,NewView),
    preserve_valid_info(MoreVertices,OldNumberOfAgents,Poss,MoreNewVertices).
    

generate_new_view([],_,_,_,[]):-!.

generate_new_view([Value|MoreValues],OldNumberOfAgents,Poss,CurrentPos,[NewValue|NewMoreValues]):-
    Modulo is CurrentPos mod OldNumberOfAgents,
    (
        (
            member(Modulo,Poss),
            NewValue = Value,
            !
        );
        (
            NewValue = '_'
        )
    ),
    NewPos is CurrentPos + 1,
    generate_new_view(MoreValues,OldNumberOfAgents,Poss,NewPos,NewMoreValues).
    


valid_simplex([],_):-!.

valid_simplex([v(Color,_)|MoreVertices],Colors):-
    member(Color,Colors),
    valid_simplex(MoreVertices,Colors).


