/*
* A view is a list of values posibly unknown (represented by '_')
*/

/*
* Predicate to fuse views
* it fuses all ith value on the views as if it where unification.
*/

fuse_views([],[]):-!.

fuse_views([View] ,View):-!.

fuse_views([View|MoreViews] ,FusedViews):-
	fuse_views(MoreViews,MoreFusedViews),
	fused_view(View, MoreFusedViews,FusedViews).

/*
* Fuse two views
* fused_view(View1,View2,FusedView)
* true if FusedView is the fusion of View1 and View2
* Fusion is the transformation from views in the simpicial complex's vertices to boolean values on epistemic model.
*/

fused_view([],[],[]):-!.

fused_view([V|More1],[V|More2],[V|MoreFused]):-
    fused_view(More1,More2,MoreFused),!.

fused_view(['_'|More1],[V2|More2],[V2|MoreFused]):-
    fused_view(More1,More2,MoreFused),!.

fused_view([V1|More1],['_'|More2],[V1|MoreFused]):-
    fused_view(More1,More2,MoreFused),!.



/*
* Given a set S and a family A of finite sbsets of S we say that A is an 
* abstract simplicial complex on S if the folowing are satisfied.
* (1) If X is an element of A and Y is contained by X, then Y is an element of A 
* (2) {v} is an elment of A for all v is S
*
* Then we can see a simplicial complex as a set of sets, 
* then we can handle them as a list of list 
* For the muddy children 
* S = { v(0,[_,0,0]),
        v(0,[_,0,1]),
        v(0,[_,1,0]),
        v(0,[_,1,1]),
        v(1,[0,_,0]),
        v(1,[0,_,1]),
        v(1,[1,_,0]),
        v(1,[1,_,1]),
        v(2,[0,0,_]),
        v(2,[0,1,_]),
        v(2,[1,0,_]),
        v(2,[1,1,_])}
* where 
* v(Color,View) represents a vertex labeled with color Color and view View
* 
* So the representation is 
[
	% All vertices
	[v(0,[_,0,0])],
    [v(0,[_,0,1])],
    [v(0,[_,1,0])],
    [v(0,[_,1,1])],
	[v(1,[0,_,0])],
    [v(1,[0,_,1])],
    [v(1,[1,_,0])],
    [v(1,[1,_,1])],
	[v(2,[0,0,_])],
    [v(2,[0,1,_])],
    [v(2,[1,0,_])],
    [v(2,[1,1,_])],
	% For each 2-simplex we have 3 1-simplices
    %000
	[v(0,[_,0,0]),v(1,[0,_,0])],
    [v(0,[_,0,0]),v(2,[0,0,_])],
    [v(1,[0,_,0]),v(2,[0,0,_])],
    [v(0,[_,0,0]),v(1,[0,_,0]),v(2,[0,0,_])],
    %001
	[v(0,[_,0,1]),v(1,[0,_,1])],
    [v(0,[_,0,1]),v(2,[0,0,_])],
    [v(1,[0,_,1]),v(2,[0,0,_])],
    [v(0,[_,0,1]),v(1,[0,_,1]),v(2,[0,0,_])],
    %010
	[v(0,[_,1,0]),v(1,[0,_,0])],
    [v(0,[_,1,0]),v(2,[0,1,_])],
    [v(1,[0,_,0]),v(2,[0,1,_])],
    [v(0,[_,1,0]),v(1,[0,_,0]),v(2,[0,1,_])],
    %011
	[v(0,[_,1,1]),v(1,[0,_,1])],
    [v(0,[_,1,1]),v(2,[0,1,_])],
    [v(1,[0,_,1]),v(2,[0,1,_])],
    [v(0,[_,1,1]),v(1,[0,_,1]),v(2,[0,1,_])],
    %100
	[v(0,[_,0,0]),v(1,[1,_,0])],
    [v(0,[_,0,0]),v(2,[1,0,_])],
    [v(1,[1,_,0]),v(2,[1,0,_])],
    [v(0,[_,0,0]),v(1,[1,_,0]),v(2,[1,0,_])],
    %101
	[v(0,[_,0,1]),v(1,[1,_,1])],
    [v(0,[_,0,1]),v(2,[1,0,_])],
    [v(1,[1,_,1]),v(2,[1,0,_])],
    [v(0,[_,0,1]),v(1,[1,_,1]),v(2,[1,0,_])],
    %110
	[v(0,[_,1,0]),v(1,[1,_,0])],
    [v(0,[_,1,0]),v(2,[1,1,_])],
    [v(1,[1,_,0]),v(2,[1,1,_])],
    [v(0,[_,1,0]),v(1,[1,_,0]),v(2,[1,1,_])],
    %111
	[v(0,[_,1,1]),v(1,[1,_,1])],
    [v(0,[_,1,1]),v(2,[1,1,_])],
    [v(1,[1,_,1]),v(2,[1,1,_])],
    [v(0,[_,1,1]),v(1,[1,_,1]),v(2,[1,1,_])]
]	
*/


/*
* facets(Complex,Facets)
* The facets on the complex represented by Complex are contained in the list Facets
*/

facets(Complex,Facets):-
	max_length(Complex,Max),
	findall(S,(member(S,Complex),length(S,Max)),Facets).

/*
* Gives an id to each facet and wraps it into an f funtor of the form f(Id,Facet)
*/

simplices_to_idsimplices([],[]):-!.

simplices_to_idsimplices([Facet|MoreFacets],[f(Id,Facet)|MoreIdFacets]):-
    length(MoreFacets,Id),
    simplices_to_idsimplices(MoreFacets,MoreIdFacets).


/* 
* max_length(List,MaxLength)
* Calculates the maximun length of the list contained in a list
*/

max_length([List|MoreLists],Length):-
	length(List,ListLength),
	max_length(MoreLists,MoreLength),
	(
        (
		    ListLength < MoreLength,
		    Length = MoreLength,
		    !
	    );
	    (
		    Length = ListLength
	    )
    ).

max_length([],-1):-!.

/*
* A facet wil be a list of vertices' id from the complex
*/
idfacet_to_world(f(Id,Facet),w(Id,FusedViews)):-
    get_views_from_facet(Facet,Views),
    fuse_views(Views,FusedViews).

/*
* get_views_from_facet(Facet,Views)
* Gets the vies from all vertices on a facet
*/
get_views_from_facet([],[]):-!.

get_views_from_facet([v(_,View)|MoreVertices],[View|MoreViews]):-
    get_views_from_facet(MoreVertices,MoreViews).

/*
* facets_to_worlds(Facets,Worlds)
*/
idfacets_to_worlds([],[]):-!.

idfacets_to_worlds([IdFacet|MoreFacets],[World|MoreWorlds]):-
    idfacet_to_world(IdFacet,World),
    idfacets_to_worlds(MoreFacets,MoreWorlds).

/*
* A model will be representated as follows 
* Worlds are lists in muddy children representing wheter ith child is muddy (1) or not (0)
* Relations and labeling on edges are represented as edge(World,Label,World)
* a Model is coposed for a list of Worlds and the list of edges
* model(Worlds,ListOfEdges)
*/

complex_to_model(Complex,Model):-
    facets(Complex,Facets),
    simplices_to_model(Facets,Model).
    
/*
* All simplices in Simplices will generate a world on Model
* The relations are built same way as usual
* simplices_to_model(Simplices,Model)
*/

simplices_to_model(Simplices,model(Worlds,Edges)):-
    simplices_to_idsimplices(Simplices,IdSimplices),
    idfacets_to_worlds(IdSimplices,Worlds),
    edges(IdSimplices,Edges).

/*
* Calcuates the edges of the given simplices
*/
edges(IdSimplices,Edges):-
    findall(
        edge(Id1,Label,Id2),
        (
            member(f(Id1,Simplex1),IdSimplices),
            member(f(Id2,Simplex2),IdSimplices),
            intersection(Simplex1,Simplex2,Intersection),
            get_colors_from_vertices(Intersection,Label),
            Label \= []
        ),
        Edges
    ).


/* Gets a list of colors from a list of vertices*/
get_colors_from_vertices([],[]):-!.

get_colors_from_vertices([v(Color,_)|MoreVertices],[Color|MoreColors]):-
    get_colors_from_vertices(MoreVertices,MoreColors).

