/*
* An action model is represented as 
* extended_action_model(ExtendedActionPoints,Edges)
* ExtendedActionPoints is a list of extended action points where
* each extended action Points has the following structure
* eap(Id,Precondition,NewInfo)
* Edges is a list of edges where each edge 
* has te same structure as in a Kripke model
* edge(SourceId,Labelling,TargetId)
* Recall that Label is a subset of all agents.
* Where Id is the name of the action point and Precondition is a formula, 
* originall, there could ba any number of preconditions but they all can be represented as a conjunction.
*/

/*
* restricted_product(Model,ActionModel,NewModel)
* true if Model x ActionModel results on NewModel
*/

extended_restricted_product(model(Worlds,Edges),extended_action_model(ActionPoints,ActionEdges),model(NewWorlds,NewEdges)):-
    extended_restricted_worlds(model(Worlds,Edges),ActionPoints,NewWorlds),
    extended_restricted_edges(NewWorlds,Edges,ActionEdges,NewEdges).

extended_restricted_worlds(model(Worlds,Edges),ActionPoints,RestrictedWorlds):-
    findall(
            w((World,ActionPoint),Values),
            (
                member(w(World,OldValues),Worlds),
                member(eap(ActionPoint,Precondition,NewValues),ActionPoints),
                check(Precondition,World,model(Worlds,Edges)),
                append(OldValues,NewValues,Values)
            ),
            RestrictedWorlds).

extended_restricted_edges(RestrictedWorlds,Edges,ActionEdges,NewEdges):-
    findall(
            edge((World1,ActionPoint1),Agents,(World2,ActionPoint2)),
            (
                member(w((World1,ActionPoint1),_),RestrictedWorlds),
                member(w((World2,ActionPoint2),_),RestrictedWorlds),
                member(edge(World1,EdgeAgents,World2),Edges),
                member(edge(ActionPoint1,ActionEdgeAgents,ActionPoint2),ActionEdges),
                intersection(EdgeAgents,ActionEdgeAgents,Agents),
                not ( Agents = [] )
            ),
            NewEdges).
    

